/*Component ms utilitzat*/
--Troba el primer component ms utilitzat i el nombre de compostos que participa
SELECT t.idComponent, t.nombrecomposts
FROM 
--Subconsulta que forma la taula de consulta
(
    --Ordena els components pel nombre de composts que participa descendentment (major participaci adalt) 
    SELECT idComponent, nombrecomposts
    FROM ComponentUtilitzat
    ORDER BY NombreComposts DESC
) t
--Sentncia que fa el TOP 1
WHERE rownum=1;

/*Producte ms produit*/
--Troba el producte ms produt amb la seva quantitat produda per un any concret
SELECT t.idProducte, t.quantitat
FROM
--Subconsulta que forma la taula de consulta
(
    --Ordena els productes descendentment en funci a la quantitat (els que tenen ms quantitat adalt)
    SELECT idProducte, quantitat
    FROM ProducteProduit
    --Limita el resultat a l'any indicat
    WHERE any_=2019
    ORDER BY quantitat DESC
) t
--Sentncia que fa el TOP 1
WHERE rownum=1;

/*Lnea ms aturada*/
--Troba la primera lnia amb ms hores aturades per l'any i mes indicats 
-- i mostra la lnia, les hores que ha estat funcionant i les hores d'aturada respecte el total d'hores d'aquell mes
SELECT t.idLinia,t.horesFuncionant,t.horesAturada
FROM
--Subconsulta que forma la taula de consulta
(
    --Agafa la lnia, l'any, mes, hores que es troba funcionant
    -- i fa el clcul de totes les hores d'aquell mes i li resta les hores que ha funcionat per obtenir les hores d'aturada
    -- ho ordena segons aquestes hores descendentment (la que t ms hores d'aturada adalt)
    SELECT idLinia,any_,mes,horesfuncionant, (1+trunc(last_day(to_date('01-'||LPAD(mes,2,'0')||'-'||any_,'DD-MM-YYYY')))-trunc(to_date('01-'||LPAD(mes,2,'0')||'-'||any_,'DD-MM-YYYY'),'MM'))*24-horesFuncionant horesAturada
    FROM LiniaAturada
    --Limita l'any i el mes
    WHERE any_=2019 AND mes=5
    ORDER BY horesAturada DESC
) t
--Sentncia que fa el TOP 1
WHERE rownum=1;

/*Capacitat emmagatzematge actual magatzem*/
--Troba el magatzem, el seu espai ocupat, la seva capacitat mxima i l'espai disponible del magatzem indicat en el moment de consulta
SELECT capacitatMagatzem.idMagatzem,capacitatMagatzem.espaiOcupat,
magatzem.capacitat,(magatzem.capacitat-capacitatMagatzem.espaiOcupat) espaiDisponible
FROM capacitatMagatzem
INNER JOIN magatzem ON
    magatzem.idMagatzem=capacitatMagatzem.idMagatzem
--Limita el magatzem
WHERE capacitatMagatzem.idMagatzem='M0001';

/*Empleats ms utilitzats*/
--Troba els 10 empleats ms utilitzats i el nombre de torns que ha realitzat cadasc per l'any indicat
SELECT t.idEmpleat,t.nombreTorns
FROM 
--Subconsulta que forma la taula de consulta
(
    --Ordena segons el nombre de torns que han realitzat els empleats descendentment (qui ha treballat ms torns adalt)
    SELECT idEmpleat,nombreTorns
    FROM empleatsUtilitzats
    --filtra per any
    WHERE any_=2019
    ORDER BY nombreTorns DESC
) t
--Sentncia que fa el TOP 10
WHERE rownum<=10;

/*Increment percentual de demanda anual*/
--Troba la demanda mitjana l'any anterior (quantitat en relaci als dies de demanda registrats) i el seu increment respecte
-- l'any anterior per un any i producte concret
SELECT t2.demandaMitjAnyAnterior,t.demandaMitjanaAnyActual,
    ((t.demandaMitjanaAnyActual-t2.demandaMitjAnyAnterior)/(t2.demandaMitjAnyAnterior))*100 incrDecrPercentAnyActual
FROM
--Subconsulta que forma la primera taula de consulta
(
    --Troba la quantitat, els dies de registre de la demana i la demanda mitjana per l'any i producte demanats
    SELECT quantitat,quantitatdies,quantitat/quantitatdies demandaMitjanaAnyActual
    FROM demandaAnual
    --Filtra per un producte i any concret
    WHERE idProducte='P0002' AND any_=2019
) t,
--Subconsulta que forma la segona taula de consulta
(
    --Troba la quantitat, els dies de registre de la demana i la demanda mitjana per l'any anterior i producte demanats
    SELECT quantitat,quantitatdies,quantitat/quantitatdies demandaMitjAnyAnterior
    FROM demandaAnual
    --Filtra per un producte i any concret (ha de ser l'anterior al de abans)
    WHERE idProducte='P0002' AND any_=2018
) t2;

/*Any menor capacitat emmagatzematge*/
--Troba l'any i l'espai ocupat on el total de l'espai ocupat entre tots els magatzems s'ha trobat al mnim
SELECT t.any_,t.espaiOcupat
FROM 
--Subconsulta que forma la taula de consulta
(
    --Ordena segons l'espai ocupat ascendentment (els registres amb menor espai ocupat adalt)
    SELECT any_,espaiOcupat
    FROM
    capacitatAnualMagatzems
    ORDER BY espaiOcupat ASC
) t 
--Sentncia que fa el TOP 1
WHERE rownum=1;

/*Increment percentual de beguda produida*/
--Troba la quantitat de litres produts d'aquest any i de l'any anterior i percentatge d'increment (o decrement) entre aquestes quantitats
SELECT t2.capacitatLitresAnyAnterior,t.capacitatLitresAnyActual,
    ((t.capacitatLitresAnyActual-t2.capacitatLitresAnyAnterior)/(t2.capacitatLitresAnyAnterior))*100 incrDecrPercentAnyActual
FROM
--Subconsulta que forma la primera taula de consulta
(
    --Troba la quantitat de litres produts de l'any de consulta
    SELECT capacitatLitres capacitatLitresAnyActual
    FROM begudaProduida
    --Filtra per l'any de consulta
    WHERE any_=EXTRACT(YEAR FROM SYSDATE)
) t,
--Subconsulta que forma la segona taula de consulta
(
    --Troba la quantitat de litres produts de l'any anterior de consulta
    SELECT capacitatLitres capacitatLitresAnyAnterior
    FROM begudaProduida
    --Filtra per l'any anterior de consulta
    WHERE any_=EXTRACT(YEAR FROM ADD_MONTHS( SYSDATE,-12))
) t2;

/*Any amb major temps de neteja*/
--Troba l'any, la lnia i les hores dedicades de neteja on s'ha dedicat ms hores per l'any indicat
SELECT t.idLinia,t.hores,any_
FROM 
--Subconsulta que forma la taula de consulta
(
    --Ordena les lnies segons les hores de neteja descendentment (les que han dedicat ms temps adalt)
    SELECT idLinia,hores,any_
    FROM netejaAnual
    --Filtra per un any concret
    WHERE any_=2019
    ORDER BY hores DESC
) t 
--Sentncia que fa el TOP 1
WHERE rownum=1;

/*Percentatge de torns incomplerts*/
--Troba el nombre de torns totals realitzats, els d'aquests que es troben complerts i el percentatge de torns incomplerts
-- d'un any indicat
SELECT tornsTotals,tornsComplerts,((tornsTotals-tornsComplerts)/tornsTotals)*100 percentatgeTornsIncomplerts
FROM tornsAnuals
--Filtra per un any indicat
WHERE any_=2019;

/*Primeres cinc empreses de distribuci de producte*/
--Troba les primeres cinc empreses que ms volum han distribut i de quin producte ha sigut
SELECT t.idEmpresa,t.idProducte,t.volum
--Subconsulta que forma la taula de consulta
FROM (
    --Ordena les lnies segons el volum descendentment (major volum distribut adalt)
    SELECT idEmpresa,idProducte,volum
    FROM empresesDistribuides
    ORDER BY volum DESC
) t 
--Sentncia que fa el TOP 5
WHERE rownum<=5;