CREATE TABLE LOG(
    idLog number(38),
    nomProc varchar(100) NOT NULL,
    dataExecucio date default sysdate NOT NULL,
    entrada varchar(500) NOT NULL,
    sortida varchar(500) NOT NULL,
    CONSTRAINT LOGPK PRIMARY KEY(idLog)
);

/*
Funci que torna el identificador nic de cada taula.
Demana el nom de la taula, el nom del camp, el primer carcter del id (per defecte buit) i la llargada (per defecte 5).
Retorna el identificador
*/
CREATE OR REPLACE FUNCTION tornaIdTaula(
    i_taula IN varchar,
    i_idCamp IN varchar,
    i_idCaracter IN varchar:='',
    i_llargada IN number:=5
) RETURN varchar
IS
    id varchar(15);
    caracters varchar(5);
    llargada number;
    ultimNombre varchar(10);
    seguentNombre number;
    lletra char(1);
    sql_stmt varchar(200);
BEGIN
    --Sentncia de SQL dinmic que monta la consulta depenent dels camps entrats
    -- Bsicament agafa l'ltim id de la taula demanada.
    sql_stmt:=
    'SELECT '||i_idCamp||'
    FROM 
        (SELECT '||i_idCamp||', rownum
        FROM TFGAdmin.' ||i_taula||' 
        ORDER BY '||i_idCamp||' DESC)
    WHERE rownum=1';
    -- Executa la sentncia de sql dinmic i guarda el resultat a id
    EXECUTE IMMEDIATE sql_stmt INTO id;
    --Si la llargada per defecte (o indicada) s superior 
    -- a la llargada de la clau actual, escull la de per defecte
    IF (i_llargada>LENGTH(id)) THEN
        llargada:=i_llargada;
    --Sin, la llargada s la de la consulta (claus anteriors)
    ELSE
        llargada:=LENGTH(id);
    END IF;
    caracters:='';
    
    --Des de 1 a la llargada del id trobat
    FOR contador IN 1..llargada
    LOOP
        --Agafa lletra per lletra
        lletra:=SUBSTR(id,contador,1);
        -- Si el carcter s alfabtic, el va afegint a la cadena caracters
        IF LENGTH(TRIM(TRANSLATE(lletra, 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ', ' '))) IS NULL THEN
            caracters:=caracters || lletra;
        -- Si el carcter s numric, l'afegeix a la cadena ultimNombre
        ELSE
            ultimNombre:=ultimNombre || lletra;
        END IF;
    END LOOP;
    --Converteix l'ltim nombre a numric i li suma 1 (segent nombre)
    seguentNombre:=CAST(ultimNombre AS NUMBER);
    seguentNombre:=seguentNombre+1;
    
    --A la cadena caracters li suma 0 la resta de la llargada que no ocupa seguentNombre
    -- Per exemple, 'F'+'00'+'11' per una llargada de 5.
    RETURN RPAD(caracters,llargada-LENGTH(seguentNombre),'0') || seguentNombre;
--Excepcions
EXCEPTION
--Quan no troba un id en el select
WHEN NO_DATA_FOUND THEN
    -- i no hi ha un carcter de inici, torna 1
    IF i_idCaracter='' THEN
        RETURN '1';
    -- si hi ha carcter de inici, el posa davant, omple de 0 la llargada-1 i l'ltim carcter s 1
    ELSE
        RETURN RPAD(i_idCaracter,i_llargada-1,'0') || '1';
    END IF;
END;
    

/*
Procediment que genera un registre a la taula LOG
Demana el nom del procediment cridat, la cadena de variables d'entrada i la cadena de variables de sortida.
*/
CREATE OR REPLACE PROCEDURE crearLOG(
    i_nomproc IN TFGAdmin.LOG.nomProc%type,
    i_entrada IN TFGAdmin.LOG.entrada%type,
    i_sortida IN TFGAdmin.LOG.sortida%type
)
IS
    id number;
BEGIN
    --Crida tornaIdTaula que retorna el segent id numric (per aquest cas)
    id:=CAST(tornaIdTaula('LOG','idLog') AS NUMBER);
    --Insereix el registre de LOG amb el nou id i els parmetres de entrada
    INSERT INTO TFGAdmin.LOG(idLog,nomProc,entrada,sortida) 
    VALUES(id,i_nomProc,i_entrada,i_sortida);
END;

/*
Procediment que genera un registre de Component
Demana la descripci del component
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE crearComponent(
    i_descripcio IN TFGAdmin.component.descripcio%type,
    RSP OUT varchar
) 
IS
    idComponent varchar(15);
    parametres varchar(500);
BEGIN
    --Per defecte, RSP s correcte
    RSP:='OK';
    --Llista de parmetres
    parametres:=i_descripcio;
    --Crida tornaIdTaula per aconseguir el segent id
    idComponent:=tornaIdTaula('component','idComponent','C');
    --Inserta el registre
    INSERT INTO TFGAdmin.component VALUES(idComponent,i_descripcio);
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
    --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que modifica un registre de Component
Demana el id i descripci del component
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE modificarComponent(
    i_idComponent IN TFGAdmin.component.idComponent%type,
    i_descripcio IN TFGAdmin.component.descripcio%type,
    RSP OUT varchar
) 
IS
    parametres varchar(500);
BEGIN
    --Per defecte, RSP s correcte
    RSP:='OK';
    --Llista de parmetres
    parametres:=i_idComponent||','||i_descripcio;
    --Actualitza el registre indicat
    UPDATE TFGAdmin.component SET
    descripcio=i_descripcio
    WHERE
    idComponent=i_idComponent;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
    --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que esborra un registre de Component
Demana el id del component
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE esborrarComponent(
    i_idComponent IN TFGAdmin.component.idComponent%type,
    RSP OUT varchar
) 
IS
    parametres varchar(500);
BEGIN
    --Per defecte, RSP s correcte
    RSP:='OK';
    --Llista de parmetres
    parametres:=i_idComponent;
    --Esborra el registre
    DELETE FROM TFGAdmin.component
    WHERE
    idComponent=i_idComponent;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
    --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment auxiliar per inserir i modificar les hores que s'ha trobat en funcionament una lnia durant una data d'inici i una de fi
Demana la lnia, la data d'inici de producci, la data d'aturada i si es tracta d'una resta d'hores (2) o d'una suma (1)
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE actualitzaLiniaAturada(
    i_linia IN TFGAdmin.produccio.idLinia%type,
    i_dataInici IN TFGAdmin.produccio.dataInici%type,
    i_dataFi IN TFGAdmin.produccio.dataFi%type,
    i_SumaResta IN number,
    RSP IN OUT varchar
)
IS
    nombreLiniaAturada number(3);
    diferenciaMesos INTEGER;
    parametres varchar(500);
BEGIN
    --Per defecte RSP s correcte
    RSP:='OK';
    --Llista de parmetres
    parametres:=i_linia||','||TO_CHAR(i_dataInici,'DD-MM-YYYY HH24:MI:SS')||','||TO_CHAR(i_dataFi,'DD-MM-YYYY HH24:MI:SS')||','||i_SumaResta;
    --Nombre enter de mesos entre la data d'inici i fi
    diferenciaMesos:=TRUNC(MONTHS_BETWEEN(i_dataFi,i_dataInici));
    --Si hi ha ms d'1 mes de diferncia
    IF diferenciaMesos>0 THEN
        --Recorre mes a mes
        FOR i in 0..diferenciaMesos
        LOOP
            --Si s el primer mes que recorre
            IF i=0 THEN
                --Guarda els registres existents d'aquell any, mes i lnia de la data inici (primer mes)
                SELECT COUNT(idLinia) INTO nombreLiniaAturada
                FROM LiniaAturada
                WHERE any_=EXTRACT(YEAR FROM i_dataInici) AND mes=EXTRACT(MONTH FROM i_dataInici) AND idLinia=i_linia;
                --Si no hi ha registres i es tracta d'una suma
                IF nombreLiniaAturada=0 AND i_SumaResta=2 THEN
                    --Insereix el registre amb l'any, mes, lnia i nombre d'hores entre el ltim dia de mes i el de data Inici
                    INSERT INTO LiniaAturada 
                    SELECT
                    EXTRACT(YEAR FROM i_dataInici),EXTRACT(MONTH FROM i_dataInici),i_linia,(TRUNC(LAST_DAY(i_dataInici))-i_dataInici)*24
                    FROM DUAL;
                --Si hi ha registres o es tracta d'una resta (ha d'existir registre)
                ELSE
                    --Actualitza la lnia amb la suma (o resta) d'hores des del final de mes fins la data d'inici
                    UPDATE LiniaAturada SET HoresFuncionant=HoresFuncionant+POWER(-1,i_SumaResta)*(TRUNC(LAST_DAY(i_dataInici))-i_dataInici)*24 WHERE any_=EXTRACT(YEAR FROM i_dataInici) AND mes=EXTRACT(MONTH FROM i_dataInici) AND idLinia=i_linia;
                END IF;
            --Si s l'ltim mes que recorre
            ELSIF i=diferenciaMesos THEN
                --Guarda els registres existents d'aquell any, mes i lnia de la data fi (ltim mes)
                SELECT COUNT(idLinia) INTO nombreLiniaAturada
                FROM LiniaAturada
                WHERE any_=EXTRACT(YEAR FROM i_dataFi) AND mes=EXTRACT(MONTH FROM i_dataFi) AND idLinia=i_linia;
                --Si no hi ha registres i es tracta d'una suma
                IF nombreLiniaAturada=0 AND i_SumaResta=2 THEN
                    --Insereix el registre amb l'any, mes, lnia i nombre d'hores entre el ltim dia de mes i el de data de Fi
                    INSERT INTO LiniaAturada 
                    SELECT
                    EXTRACT(YEAR FROM i_dataFi),EXTRACT(MONTH FROM i_dataFi),i_linia,(i_dataFi-trunc(i_dataFi, 'MM'))*24
                    FROM DUAL;
                --Si hi ha registres o es tracta d'una resta (ha d'existir registre)
                ELSE
                    --Actualitza la lnia amb la suma (o resta) d'hores des del final de mes fins la data de fi
                    UPDATE LiniaAturada SET HoresFuncionant=HoresFuncionant+POWER(-1,i_SumaResta)*(i_dataFi-trunc(i_dataFi, 'MM'))*24 WHERE any_=EXTRACT(YEAR FROM i_dataFi) AND mes=EXTRACT(MONTH FROM i_dataFi) AND idLinia=i_linia;
                END IF;
            --Si s un mes d'entremig
            ELSE
                --Guarda els registres existents d'aquell any, mes i lnia des de la data d'inici (suma el nombre de mesos)
                SELECT COUNT(idLinia) INTO nombreLiniaAturada
                FROM LiniaAturada
                WHERE any_=EXTRACT(YEAR FROM ADD_MONTHS(i_dataInici,i)) AND mes=EXTRACT(MONTH FROM ADD_MONTHS(i_dataInici,i)) AND idLinia=i_linia;
                --Si no hi ha registres i es tracta d'una suma
                IF nombreLiniaAturada=0 AND i_SumaResta=2 THEN
                    --Insereix el registre amb l'any, mes, lnia i nombre d'hores entre el ltim dia de mes i el primer dia del mes
                    INSERT INTO LiniaAturada 
                    SELECT
                    EXTRACT(YEAR FROM ADD_MONTHS(i_dataInici,i)),EXTRACT(MONTH FROM ADD_MONTHS(i_dataInici,i)),i_linia,(1+trunc(last_day(ADD_MONTHS(i_dataInici,i)))-trunc(ADD_MONTHS(i_dataInici,i),'MM'))*24
                    FROM DUAL;
                --Si hi ha registres o es tracta d'una resta (ha d'existir registre)
                ELSE
                    --Actualitza la lnia amb la suma (o resta) d'hores des del final de mes fins al inici de mes
                    UPDATE LiniaAturada SET HoresFuncionant=HoresFuncionant+POWER(-1,i_SumaResta)*(1+trunc(last_day(ADD_MONTHS(i_dataInici,i)))-trunc(ADD_MONTHS(i_dataInici,i),'MM'))*24 WHERE any_=EXTRACT(YEAR FROM ADD_MONTHS(i_dataInici,i)) AND mes=EXTRACT(MONTH FROM ADD_MONTHS(i_dataInici,i)) AND idLinia=i_linia;
                END IF;
            END IF;
        END LOOP;
    --Si no hi ha ms d'un mes de diferncia
    ELSE
        --Guarda els registres existents d'aquell any, mes i lnia de la data inici (primer mes)
        SELECT COUNT(idLinia) INTO nombreLiniaAturada
        FROM LiniaAturada
        WHERE any_=EXTRACT(YEAR FROM i_dataInici) AND mes=EXTRACT(MONTH FROM i_dataInici) AND idLinia=i_linia;
        --Si no hi ha registres i es tracta d'una suma
        IF nombreLiniaAturada=0 AND i_SumaResta=2 THEN
            --Insereix el registre amb l'any, mes, lnia i nombre d'hores entre la data de fi i la data d'inici
            INSERT INTO LiniaAturada
            SELECT
            EXTRACT(YEAR FROM i_dataInici),EXTRACT(MONTH FROM i_dataInici),i_linia,(i_dataFi-i_dataInici)*24
            FROM DUAL;
        --Si hi ha registres o es tracta d'una resta (ha d'existir registre)
        ELSE
            --Actualitza la lnia amb la suma (o resta) d'hores des de la data de fi fins la data d'inici
            UPDATE LiniaAturada SET HoresFuncionant=HoresFuncionant+POWER(-1,i_SumaResta)*(i_dataFi-i_dataInici)*24 WHERE any_=EXTRACT(YEAR FROM i_dataInici) AND mes=EXTRACT(MONTH FROM i_dataInici) AND idLinia=i_linia;
        END IF;
    END IF;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que crea un compost (relaciona component amb producte)
Demana el producte, el component del producte, la quantitat i la unitat de mesura que utilitza
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE crearCompost(
    i_producte IN TFGAdmin.compost.idProducte%type,
    i_component IN TFGAdmin.compost.idComponent%type,
    i_quantitat IN TFGAdmin.compost.quantitat%type,
    i_unitatMesura IN TFGAdmin.compost.unitatMesura%type,
    RSP OUT varchar
) 
IS
    nombreCompostComponent number(10);
    parametres varchar(500);
BEGIN
    --Per defecte RSP s correcte
    RSP:='OK';
    --Llista de parmetres
    parametres:=i_producte||','||i_component||','||i_quantitat||','||i_unitatMesura;
    --Guarda el nombre de registres actuals que est sent utilitzat aquest component
    SELECT COUNT(idComponent) INTO nombreCompostComponent
    FROM TFGAdmin.componentUtilitzat
    WHERE idComponent=i_component;
    --Insereix el registre a compost
    INSERT INTO TFGAdmin.compost VALUES(i_producte,i_component,i_quantitat,i_unitatMesura);
    --Si no hi ha registres com a component utilitzat
    IF nombreCompostComponent=0 THEN
        --Insereix un nou registre com a component utilitzat
        INSERT INTO TFGAdmin.componentUtilitzat VALUES(i_component,1);
    --Si hi ha registres
    ELSE
        --Actualitza la taula sumant-li un al nombre total de productes que utilitzen el component
        UPDATE TFGAdmin.componentUtilitzat SET nombreComposts=nombreComposts+1 WHERE idComponent=i_component;
    END IF;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que modifica un registre de Compost
Demana el producte, el component d'aquest producte, la quantitat i la unitat de mesura de la quantitat
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE modificarCompost(
    i_producte IN TFGAdmin.compost.idProducte%type,
    i_component IN TFGAdmin.compost.idComponent%type,
    i_quantitat IN TFGAdmin.compost.quantitat%type,
    i_unitatMesura IN TFGAdmin.compost.unitatMesura%type,
    RSP OUT varchar
) 
IS
    nombreCompostComponent number(10);
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_producte||','||i_component||','||i_quantitat||','||i_unitatMesura;
    --Actualitza el registre indicat
    UPDATE TFGAdmin.compost SET quantitat=i_quantitat,unitatMesura=i_unitatMesura WHERE idProducte=i_producte AND idComponent=i_component;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que esborra el registre de compost d'un producte
Demana el producte i el seu component
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE esborrarCompost(
    i_producte IN TFGAdmin.compost.idProducte%type,
    i_component IN TFGAdmin.compost.idComponent%type,
    RSP OUT varchar
) 
IS
    nombreCompostComponent number(10);
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_producte||','||i_component;
    --Esborra el registre de Compost
    DELETE TFGAdmin.compost WHERE idProducte=i_producte AND idComponent=i_component;
    --Resta una unitat el nombre de composts que participa el component
    UPDATE TFGAdmin.componentUtilitzat SET nombreComposts=nombreComposts-1 WHERE idComponent=i_component;
    --Elimina aquells registres que hagin quedat a 0
    DELETE TFGAdmin.componentUtilitzat WHERE nombreComposts=0;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que crea registres a producci
Demana el producte produt, la lnia on es produeix, la data d'inici i la de fi de la producci i la quantitat fabricada
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE crearProduccio(
    i_producte IN TFGAdmin.produccio.idProducte%type,
    i_linia IN TFGAdmin.produccio.idLinia%type,
    i_dataInici IN TFGAdmin.produccio.dataInici%type,
    i_dataFi IN TFGAdmin.produccio.dataFi%type,
    i_quantitatFabricada IN TFGAdmin.produccio.quantitatFabricada%type,
    RSP OUT varchar
) 
IS
    anyFi integer;
    mesFi integer;
    nombreProducteAny number(3);
    diferenciaMesos number(3);
    nombreLiniaAturada number(3);
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_producte||','||i_linia||','||TO_CHAR(i_dataInici,'DD-MM-YYYY HH24:MI:SS')||','||TO_CHAR(i_dataFi,'DD-MM-YYYY HH24:MI:SS')||','||i_quantitatFabricada;
    --Any de fi de la producci
    anyFi:=EXTRACT(YEAR FROM i_dataFi);
    --Nombre de registres del producte en aquell any que s'han produt
    SELECT COUNT(idProducte) INTO nombreProducteAny
    FROM TFGAdmin.producteProduit
    WHERE any_=anyFi AND idProducte=i_producte;
    --Insereix el registre a produccio
    INSERT INTO TFGAdmin.produccio VALUES(i_producte,i_linia,i_dataInici,i_dataFi,i_quantitatFabricada);
    
    /*Producte ms produit*/
    --Si no hi ha registres
    IF nombreProducteAny=0 THEN
        --Insereix un registre amb l'any, producte i quantitat que s'han fabricat
        INSERT INTO TFGAdmin.producteProduit VALUES(anyFi,i_producte,i_quantitatFabricada);
    --Si hi ha registres
    ELSE   
        --Actualitza el registre sumant la nova quantitat produda
        UPDATE TFGAdmin.producteProduit SET quantitat=quantitat+i_quantitatFabricada WHERE any_=anyFi AND idProducte=i_producte;
    END IF;
    
    /*Linia Aturada*/
    --Crea un registre amb les hores de la lnia
    actualitzaLiniaAturada(i_linia,i_dataInici,i_dataFi,2,RSP);
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que modifica un registre de produccio
Demana el producte produt, la lnia on es produeix, la data d'inici i la de fi de la producci i la quantitat fabricada
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE modificarProduccio(
    i_producte IN TFGAdmin.produccio.idProducte%type,
    i_linia IN TFGAdmin.produccio.idLinia%type,
    i_dataInici IN TFGAdmin.produccio.dataInici%type,
    i_dataFi IN TFGAdmin.produccio.dataFi%type,
    i_quantitatFabricada IN TFGAdmin.produccio.quantitatFabricada%type,
    RSP OUT varchar
) 
IS
    anyFi integer;
    anyFiAnterior integer;
    nombreProducteAny number(3);
    quantitatAnterior TFGAdmin.produccio.quantitatFabricada%type;
    dataFiAnterior TFGAdmin.produccio.dataFi%type;
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_producte||','||i_linia||','||TO_CHAR(i_dataInici,'DD-MM-YYYY HH24:MI:SS')||','||TO_CHAR(i_dataFi,'DD-MM-YYYY HH24:MI:SS')||','||i_quantitatFabricada;
    --Obt l'any de la data de fi
    anyFi:=EXTRACT(YEAR FROM i_dataFi);
    --Guarda l'anterior quantitat fabricada amb la seva data de fi
    SELECT quantitatFabricada,dataFi INTO quantitatAnterior,dataFiAnterior
    FROM TFGAdmin.produccio
    WHERE idProducte=i_producte AND idLinia=i_linia AND dataInici=i_dataInici;
    --Obt l'any de la data de fi anterior
    anyFiAnterior:=EXTRACT(YEAR FROM dataFiAnterior);
    --Actualitza el registre de produccio
    UPDATE TFGAdmin.produccio SET dataFi=i_dataFi,quantitatFabricada=i_quantitatFabricada WHERE idProducte=i_producte AND idLinia=i_linia AND dataInici=i_dataInici;
    
    /*Producte ms produit*/
    --Guarda els registres de producte produt pel nou any
    SELECT COUNT(idProducte) INTO nombreProducteAny
    FROM TFGAdmin.producteProduit
    WHERE any_=anyFi AND idProducte=i_producte;
    --Actualitza el registre anterior treient la quantitat que s'ha produt
    UPDATE TFGAdmin.producteProduit SET quantitat=quantitat-quantitatAnterior WHERE any_=anyFiAnterior AND idProducte=i_producte;
    --Si no hi ha registres al nou any
    IF nombreProducteAny=0 THEN
        --Crea el nou registre
        INSERT INTO TFGAdmin.producteProduit VALUES(anyFi,i_producte,i_quantitatFabricada);
    --Si hi ha registres al nou any
    ELSE
        --Li suma la nova quantitat
        UPDATE TFGAdmin.producteProduit SET quantitat=quantitat+i_quantitatFabricada WHERE any_=anyFi AND idProducte=i_producte;
    END IF;
    --Esborra aquells registres que hagin quedat a 0
    DELETE TFGAdmin.producteProduit WHERE quantitat=0;
    
    /*Linia aturada*/
    --Treu les hores de producci anterior
    actualitzaLiniaAturada(i_linia,i_dataInici,dataFiAnterior,1,RSP);
    --Suma les noves hores de producci
    actualitzaLiniaAturada(i_linia,i_dataInici,i_dataFi,2,RSP);
    --Esborra les lnies sense hores de funcionament
    DELETE TFGAdmin.liniaAturada WHERE horesFuncionant=0;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que esborra un registre de produccio
Demana el producte, la lnia i la data d'inici (claus)
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE esborrarProduccio(
    i_producte IN TFGAdmin.produccio.idProducte%type,
    i_linia IN TFGAdmin.produccio.idLinia%type,
    i_dataInici IN TFGAdmin.produccio.dataInici%type,
    RSP OUT varchar
) 
IS
    anyFi integer;
    quantitatAnterior TFGAdmin.produccio.quantitatFabricada%type;
    dataFi TFGAdmin.produccio.dataFi%type;
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_producte||','||i_linia||','||TO_CHAR(i_dataInici,'DD-MM-YYYY HH24:MI:SS');
    --Guarda la quantitat i data de fi anterior del registre de produccio
    SELECT quantitatFabricada,dataFi INTO quantitatAnterior,dataFi
    FROM TFGAdmin.produccio
    WHERE idProducte=i_producte AND idLinia=i_linia AND dataInici=i_dataInici;
    --Obt l'any de la data de fi
    anyFi:=EXTRACT(YEAR FROM dataFi);
    --Esborra el registre de produccio
    DELETE TFGAdmin.produccio WHERE idProducte=i_producte AND idLinia=i_linia AND dataInici=i_dataInici;
    
    /*Producte ms produit*/
    --Resta la quantitat anterior al registre de producteProduit
    UPDATE TFGAdmin.producteProduit SET quantitat=quantitat-quantitatAnterior WHERE any_=anyFi AND idProducte=i_producte;
    --Esborra els registres a 0
    DELETE TFGAdmin.producteProduit WHERE quantitat=0;
    
    /*Linia aturada*/
    --Resta les hores de producci a la lnia corresponent
    actualitzaLiniaAturada(i_linia,i_dataInici,dataFi,1,RSP);
    --Elimina els registres a 0
    DELETE TFGAdmin.liniaAturada WHERE horesFuncionant=0;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Per qualsevol excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que crea un registre de Stock
Demana el sistema d'embalatge, el magatzem on es guarda, la data del registre,
el volum que ocupa, la quantitat del sistema, el transport utilitzat i l'empresa (si s que surt)
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE crearStock(
    i_idSistema TFGAdmin.stock.idSistema%type,
    i_idMagatzem TFGAdmin.stock.idMagatzem%type,
    i_data TFGAdmin.stock.data%type,
    i_volum TFGAdmin.stock.volum%type,
    i_quantitat TFGAdmin.stock.quantitat%type,
    i_idTransport TFGAdmin.stock.idTransport%type,
    i_idEmpresa TFGAdmin.stock.idEmpresa%type,
    RSP OUT varchar
)
IS
    producteSistema TFGAdmin.producte.idProducte%type;
    existeixEmpresaDistribuida number(1);
    capacitatEnvasML TFGAdmin.envas.capacitat%type;
    existeixBegudaAnual number(1);
    existeixCapacitatAnual number(1);
    espaiOcupat TFGAdmin.capacitatMagatzem.espaiOcupat%type;
    excesEspai EXCEPTION;
    espaiMagatzem TFGAdmin.magatzem.capacitat%type;
    registreMagatzem number(1);
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_idSistema||','||i_idMagatzem||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS')||','||i_volum||','||i_quantitat||','||i_idTransport||','||i_idEmpresa;
    --Guarda la capacitat total del magatzem
    SELECT capacitat INTO espaiMagatzem
    FROM magatzem
    WHERE idMagatzem=i_idMagatzem;
    --Guarda els registres que hi ha en capacitatMagatzem
    SELECT COUNT(*) INTO registreMagatzem
    FROM capacitatMagatzem
    WHERE idMagatzem=i_idMagatzem;
    
    /*capacitat Magatzem*/
    --Si no hi ha registre
    IF registreMagatzem=0 THEN
        --Si el volum inserit sobrepassa al espai disponible
        IF i_volum>espaiMagatzem THEN
            --Crida l'excepci excesEspai
            RAISE excesEspai;
        END IF;
        --En cas contrari, insereix el registre a la capacitat magatzem
        INSERT INTO capacitatMagatzem VALUES(i_idMagatzem,i_volum);
    --Si hi ha registre
    ELSE
        --Guarda l'espai ocupat actual del magatzem
        SELECT espaiOcupat INTO espaiOcupat
        FROM capacitatMagatzem
        WHERE idMagatzem=i_idMagatzem;
        --Si afegint el nou espai ocupat supera el lmit del magatzem
        IF (espaiOcupat+i_volum)>espaiMagatzem THEN
            --Crida l'excepci excesEspai
            RAISE excesEspai;
        END IF;
        --En cas contrari, suma el nou espai a l'espai ocupat del magatzem
        UPDATE capacitatMagatzem SET espaiOcupat=espaiOcupat+i_volum WHERE idMagatzem=i_idMagatzem;
    END IF;
    
    /*Capacitat Anual Magatzem*/
    --Guarda el nombre de registres de la capacitatAnualMagatzems
    SELECT COUNT(*) INTO existeixCapacitatAnual
    FROM capacitatAnualMagatzems
    WHERE any_=EXTRACT(YEAR FROM i_data);
    --Si no existeix registre
    IF existeixCapacitatAnual=0 THEN
        --El insereix
        INSERT INTO capacitatAnualMagatzems VALUES(EXTRACT(YEAR FROM i_data),i_volum,i_volum);
    --Si existeix el registre
    ELSE
        --Actualitza l'espai ocupat i el seu mxim si resulta que l'espaiOcupat ms el nou volum s superior al guardat
        UPDATE capacitatAnualMagatzems SET espaiOcupat=espaiOcupat+i_volum,
        espaiMaximOcupat=(CASE WHEN espaiOcupat+i_volum>espaiMaximOcupat THEN espaiOcupat+i_volum ELSE espaiMaximOcupat END)
        WHERE any_=EXTRACT(YEAR FROM i_data);
    END IF;
    
    /*Beguda produida anualment*/
    --Guarda la capacitat de l'envs i el producte del sistema
    SELECT Envas.capacitat,Producte.idProducte INTO capacitatEnvasML,producteSistema
    FROM SistemaEmbalatge
    INNER JOIN Producte ON
        Producte.idProducte=SistemaEmbalatge.idProducte
    INNER JOIN Envas ON
        Producte.idEnvas=Envas.idEnvas
    WHERE SistemaEmbalatge.idSistema=i_idSistema;
    --Si el volum entrant s superior a 0
    IF i_volum>0 THEN
        --Guarda els registres de begudaProduida d'aquell any
        SELECT COUNT(*) INTO existeixBegudaAnual
        FROM begudaProduida
        WHERE any_=EXTRACT(YEAR FROM i_data);
        --Si no hi ha registre
        IF existeixBegudaAnual=0 THEN
            --Insereix el registre calculant els litres de beguda segons la quantitat i capacitat de l'envs (mL)
            INSERT INTO begudaProduida VALUES(EXTRACT(YEAR FROM i_data),i_quantitat*capacitatEnvasML/1000);
        --Si hi ha registre
        ELSE
            --Suma a la capacitat produda els litres calculats de la no capacitat
            UPDATE begudaProduida SET capacitatLitres=capacitatLitres+i_quantitat*capacitatEnvasML/1000
            WHERE any_=EXTRACT(YEAR FROM i_data);
        END IF;
    END IF;
    
    /*Empreses Distribuides*/
    --Si el parmetre d'empresa est ple
    IF i_idEmpresa IS NOT NULL THEN
        --Guarda els registres que hi ha d'aquella empresa i producte
        SELECT COUNT(idEmpresa) INTO existeixEmpresaDistribuida
        FROM empresesDistribuides
        WHERE idEmpresa=i_idEmpresa AND idProducte=producteSistema;
        --Si no existeix registre
        IF existeixEmpresaDistribuida=0 THEN
            --Insereix el registre posant la quantitat en positiu (la quantitat entrada ha de ser negativa perqu surt del Stock)
            INSERT INTO empresesDistribuides VALUES(i_idEmpresa,producteSistema,-i_quantitat*capacitatEnvasML/1000);
        --Si existeix registre
        ELSE
            --Suma al volum del registre el volum distribuit
            UPDATE empresesDistribuides SET volum=volum-i_quantitat*capacitatEnvasML/1000 
            WHERE idEmpresa=i_idEmpresa AND idProducte=producteSistema;
        END IF;
    END IF;
    
    /*Stock*/
    --Finalment, si tot ha funcionat, crea el registre a Stock
    INSERT INTO Stock VALUES(i_idSistema,i_idMagatzem,i_data,i_volum,i_quantitat,i_idTransport,i_idEmpresa);
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Excepcions
EXCEPTION
--Excepci que es crida si es passa de l'espai del magatzem
WHEN excesEspai THEN
    --Error personalitzat
    RSP:='ERROR'|| ':' ||'espai insuficient al magatzem '||i_idMagatzem;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que modifica un registre de Stock
Demana el sistema d'embalatge, el magatzem on es guarda, la data del registre,
el volum que ocupa, la quantitat del sistema, el transport utilitzat i l'empresa (si s que surt)
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE modificarStock(
    i_idSistema TFGAdmin.stock.idSistema%type,
    i_idMagatzem TFGAdmin.stock.idMagatzem%type,
    i_data TFGAdmin.stock.data%type,
    i_volum TFGAdmin.stock.volum%type,
    i_quantitat TFGAdmin.stock.quantitat%type,
    i_idTransport TFGAdmin.stock.idTransport%type,
    i_idEmpresa TFGAdmin.stock.idEmpresa%type,
    RSP OUT varchar
)
IS
    producteSistema TFGAdmin.producte.idProducte%type;
    quantitatAnterior TFGAdmin.stock.quantitat%type;
    capacitatEnvasML TFGAdmin.envas.capacitat%type;
    volumAnterior TFGAdmin.stock.volum%type;
    espaiOcupat TFGAdmin.capacitatMagatzem.espaiOcupat%type;
    excesEspai EXCEPTION;
    espaiMagatzem TFGAdmin.magatzem.capacitat%type;
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_idSistema||','||i_idMagatzem||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS')||','||i_volum||','||i_quantitat||','||i_idTransport||','||i_idEmpresa;
    --Guarda la capacitat mxima del magatzem implicat
    SELECT capacitat INTO espaiMagatzem
    FROM magatzem
    WHERE idMagatzem=i_idMagatzem;
    --Guarda el volum i quantitat anterior del registre modificat
    SELECT volum,quantitat INTO volumAnterior,quantitatAnterior
    FROM Stock
    WHERE idSistema=i_idSistema AND idMagatzem=i_idMagatzem AND data=i_data;
    
    /*capacitat Magatzem*/
    --Guarda l'espai ocupat del registre anterior del magatzem
    SELECT espaiOcupat INTO espaiOcupat
    FROM capacitatMagatzem
    WHERE idMagatzem=i_idMagatzem;
    --Si l'espai anterior menys el volum anterior ms el nou volum s major a l'espai mxim
    IF (espaiOcupat-volumAnterior+i_volum)>espaiMagatzem THEN
        --Crida l'excepci excesEspai
        RAISE excesEspai;
    END IF;
    --Si passa, actualitza l'espai ocupat pel nou
    UPDATE capacitatMagatzem SET espaiOcupat=espaiOcupat-volumAnterior+i_volum WHERE idMagatzem=i_idMagatzem;
    --Esborra els registres que quedin a 0
    DELETE capacitatMagatzem WHERE espaiOcupat=0;
    
    /*Capacitat Anual Magatzem*/
    --Actualitza l'espai ocupat anual i el seu mxim si dna el cas que la modificaci fa que sigui superior
    UPDATE capacitatAnualMagatzems SET espaiOcupat=espaiOcupat-volumAnterior+i_volum,
        espaiMaximOcupat=(CASE WHEN espaiOcupat-volumAnterior+i_volum>espaiMaximOcupat THEN espaiOcupat-volumAnterior+i_volum ELSE espaiMaximOcupat END)
    WHERE any_=EXTRACT(YEAR FROM i_data);
    --Esborra els registres a 0
    DELETE capacitatAnualMagatzems WHERE espaiMaximOcupat=0;
    
    /*Beguda produida anualment*/
    --Guarda la capacitat de l'envs i el producte del sistema
    SELECT Envas.capacitat,Producte.idProducte INTO capacitatEnvasML,producteSistema
    FROM SistemaEmbalatge
    INNER JOIN Producte ON
        Producte.idProducte=SistemaEmbalatge.idProducte
    INNER JOIN Envas ON
        Producte.idEnvas=Envas.idEnvas
    WHERE SistemaEmbalatge.idSistema=i_idSistema;
    --Si el volum s positiu
    IF i_volum>0 THEN
        --Actualitza els litres treient la quantitat anterior i reemplaant-la per la nova quantitat
        UPDATE begudaProduida SET capacitatLitres=capacitatLitres+(i_quantitat-quantitatAnterior)*capacitatEnvasML/1000
        WHERE any_=EXTRACT(YEAR FROM i_data);
        --Esborra els registres negatius o a 0 (tot i que no hi haurien d'haver de negatius)
        DELETE begudaProduida WHERE capacitatLitres<=0;
    END IF;
    
    /*Empreses Distribuides*/
    --Si hi ha informada l'empresa
    IF i_idEmpresa IS NOT NULL THEN
        --Actualitza el registre amb el nou volum (li treu l'antic i suma el nou)
        UPDATE empresesDistribuides SET volum=volum-(i_quantitat-quantitatAnterior)*capacitatEnvasML/1000 
        WHERE idEmpresa=i_idEmpresa AND idProducte=producteSistema;
        --Esborra els registres a 0
        DELETE empresesDistribuides WHERE volum=0;
    END IF;
 
    /*Stock*/
    --Si tot ha anat b, actualitza el registre de Stock amb les noves dades
    UPDATE Stock SET volum=i_volum, quantitat=i_quantitat,idTransport=i_idTransport,idEmpresa=i_idEmpresa
    WHERE idSistema=i_idSistema AND idMagatzem=i_idMagatzem AND data=i_data;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
    
--Excepcions
EXCEPTION
--Excepci que es crida si es passa de l'espai del magatzem
WHEN excesEspai THEN
    --Error personalitzat
    RSP:='ERROR'|| ':' ||'espai insuficient al magatzem '||i_idMagatzem;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que esborra un registre de Stock
Demana el sistema d'embalatge, el magatzem on es guarda i la data del registre (claus)
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE esborrarStock(
    i_idSistema TFGAdmin.stock.idSistema%type,
    i_idMagatzem TFGAdmin.stock.idMagatzem%type,
    i_data TFGAdmin.stock.data%type,
    RSP OUT varchar
)
IS
    empresaAnterior TFGAdmin.stock.idEmpresa%type;
    producteSistema TFGAdmin.producte.idProducte%type;
    quantitatAnterior TFGAdmin.stock.quantitat%type;
    capacitatEnvasML TFGAdmin.envas.capacitat%type;
    volumAnterior TFGAdmin.stock.volum%type;
    espaiOcupat TFGAdmin.capacitatMagatzem.espaiOcupat%type;
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_idSistema||','||i_idMagatzem||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS');
    --Guarda el volum del registre anterior
    SELECT volum,quantitat INTO volumAnterior,quantitatAnterior
    FROM Stock
    WHERE idSistema=i_idSistema AND idMagatzem=i_idMagatzem AND data=i_data;
    
    /*capacitat Magatzem*/
    --Guarda l'espai ocupat del registre anterior
    SELECT espaiOcupat INTO espaiOcupat
    FROM capacitatMagatzem
    WHERE idMagatzem=i_idMagatzem;
    --Treu el volum anterior de l'espai ocupat del registre anterior
    UPDATE capacitatMagatzem SET espaiOcupat=espaiOcupat-volumAnterior WHERE idMagatzem=i_idMagatzem;
    --Esborra els registres a 0
    DELETE capacitatMagatzem WHERE espaiOcupat=0;
    
    /*Capacitat Anual Magatzem*/
    --Elimina el volum anterior i li treu del mxim
    UPDATE capacitatAnualMagatzems SET espaiOcupat=espaiOcupat-volumAnterior,espaiMaximOcupat=espaiMaximOcupat-volumAnterior 
    WHERE any_=EXTRACT(YEAR FROM i_data);
    --Esborra els registres a 0
    DELETE capacitatAnualMagatzems WHERE espaiMaximOcupat=0;
    
    /*Beguda produida anualment*/
    --Guarda la capacitat i el producte del sistema
    SELECT Envas.capacitat,Producte.idProducte INTO capacitatEnvasML,producteSistema
    FROM SistemaEmbalatge
    INNER JOIN Producte ON
        Producte.idProducte=SistemaEmbalatge.idProducte
    INNER JOIN Envas ON
        Producte.idEnvas=Envas.idEnvas
    WHERE SistemaEmbalatge.idSistema=i_idSistema;
    --En cas que el volum fos positiu
    IF volumAnterior>0 THEN
        --Li resta a la capacitat produda d'aquell any
        UPDATE begudaProduida SET capacitatLitres=capacitatLitres-quantitatAnterior*capacitatEnvasML/1000
        WHERE any_=EXTRACT(YEAR FROM i_data);
        --Esborra els registres a 0
        DELETE begudaProduida WHERE capacitatLitres<=0;
    END IF;
    
    /*Empreses Distribuides*/
    --Si tenia registre a empresa
    IF empresaAnterior IS NOT NULL THEN
        --Li resta la quantitat anterior que no ha distribuit (la quantitat anterior est guardada com a negatiu)
        UPDATE empresesDistribuides SET volum=volum+quantitatAnterior*capacitatEnvasML/1000 
        WHERE idEmpresa=empresaAnterior AND idProducte=producteSistema;
        --Treu els registres a 0
        DELETE empresesDistribuides WHERE volum=0;
    END IF;
    
    /*Stock*/
    --Si tot ha anat b, elimina el registre de Stock
    DELETE Stock
    WHERE idSistema=i_idSistema AND idMagatzem=i_idMagatzem AND data=i_data;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
    
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;
/*
Procediment que crea un registre d'assignaci entre empleat i torn
Demana l'empleat, el torn assignat i la data d'assignaci
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE crearAssignacio(
    i_idEmpleat TFGAdmin.assignacio.idEmpleat%type,
    i_idTorn TFGAdmin.assignacio.idTorn%type,
    i_data TFGAdmin.assignacio.data%type,
    RSP OUT varchar
)
IS
    capacitatPersones TFGAdmin.torn.capacitatPersones%type;
    tornsActuals number(5);
    personesActuals TFGAdmin.torn.capacitatPersones%type;
    existeixTornAnual number(1);
    empleatUtilitzat number(1);
    parametres varchar(500);
    comptador number(1);
BEGIN
    RSP:='OK';
    parametres:=i_idEmpleat||','||i_idTorn||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS');
    --Guarda el nombre de registres que participa l'empleat aquell any dels torns que ha fet
    SELECT COUNT(*) INTO empleatUtilitzat
    FROM empleatsUtilitzats
    WHERE idEmpleat=i_idEmpleat AND any_=EXTRACT(YEAR FROM i_data);
    --Insereix el registre a assignacio
    INSERT INTO assignacio VALUES(i_idEmpleat,i_idTorn,i_data);
    
    /*empleats utilitzats*/
    --Si no hi ha registres
    IF empleatUtilitzat=0 THEN
        --Crea el registre amb un comptador de 1
        INSERT INTO empleatsUtilitzats VALUES(EXTRACT(YEAR FROM i_data),i_idEmpleat,1);
    --Si existeix el registre
    ELSE
        --Li suma una unitat al comptador
        UPDATE empleatsUtilitzats SET nombreTorns=nombreTorns+1 WHERE any_=EXTRACT(YEAR FROM i_data) AND idEmpleat=i_idEmpleat;
    END IF;
    
    /*torns anuals*/
    --Guarda el nombre mxim de persones del torn
    SELECT capacitatPersones INTO capacitatPersones
    FROM torn
    WHERE idTorn=i_idtorn;
    --Guarda el nombre de torns
    SELECT COUNT(idTorn) INTO tornsActuals
    FROM torn;
    --Guarda el nombre d'empleats assignats d'aquell torn
    SELECT COUNT(idEmpleat) INTO personesActuals
    FROM Assignacio
    WHERE idTorn=i_idTorn;
    --Guarda els registres de torns anuals
    SELECT COUNT(any_) INTO existeixTornAnual
    FROM tornsAnuals
    WHERE any_=EXTRACT(YEAR FROM i_data);
    --Si la capacitat mxima s la mateixa que les persones que hi han assignades, el comptador s 1
    IF capacitatPersones=personesActuals THEN
        comptador:=1;
    --Sin, el comptador s 0
    ELSE
        comptador:=0;
    END IF;
    
    --si no existeix registre
    IF existeixTornAnual=0 THEN
        --Crea el registre amb el comptador de torns complerts
        INSERT INTO tornsAnuals VALUES(EXTRACT(YEAR FROM i_data),tornsActuals,comptador);
    --si existeix el registre
    ELSE
        --Li suma la quantitat del comptador als torns complerts
        UPDATE tornsAnuals SET tornsComplerts=tornsComplerts+comptador WHERE any_=EXTRACT(YEAR FROM i_data);
    END IF;
    
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que esborra un registre d'assignaci de torn d'un empleat
Demana l'empleat, el torn assignat i la data d'assignaci
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE esborrarAssignacio(
    i_idEmpleat TFGAdmin.assignacio.idEmpleat%type,
    i_idTorn TFGAdmin.assignacio.idTorn%type,
    i_data TFGAdmin.assignacio.data%type,
    RSP OUT varchar
)
IS
    capacitatPersones TFGAdmin.torn.capacitatPersones%type;
    personesAnteriors TFGAdmin.torn.capacitatPersones%type;
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_idEmpleat||','||i_idTorn||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS');
    --Guarda els registres d'assignaci d'aquell torn abans de l'esborrat
    SELECT COUNT(idEmpleat) INTO personesAnteriors
    FROM Assignacio
    WHERE idTorn=i_idTorn;
    
    /*Assignacio*/
    --Esborra el registre d'assignaci
    DELETE assignacio WHERE idEmpleat=i_idEmpleat AND idTorn=i_idTorn AND data=i_data;
    
    /*empleats utilitzats*/
    --Treu un comptador al nombre de torns d'aquell empleat d'aquell any
    UPDATE empleatsUtilitzats SET nombreTorns=nombreTorns-1 WHERE any_=EXTRACT(YEAR FROM i_data) AND idEmpleat=i_idEmpleat;
    --Esborra els registres a 0
    DELETE empleatsUtilitzats WHERE nombreTorns=0;
    
    /*torns anuals*/
    --Guarda el nombre mxim de persones per torn
    SELECT capacitatPersones INTO capacitatPersones
    FROM torn
    WHERE idTorn=i_idtorn;
    --Si la capacitat mxima s la mateixa que el nombre de persones d'abans d'esborrar el registre
    IF capacitatPersones=personesAnteriors THEN
        --Treu un comptador al nombre de torns complerts d'aquell any
        UPDATE tornsAnuals SET tornsComplerts=tornsComplerts-1 WHERE any_=EXTRACT(YEAR FROM i_data);
    END IF;
    --Treu els registres a 0
    DELETE tornsAnuals WHERE tornsComplerts=0;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que crea un registre de demanda
Demana el producte, la zona, la data i la quantitat de la demanda
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE crearDemanda(
    i_idProducte IN TFGAdmin.demanda.idProducte%type,
    i_idZona IN TFGAdmin.demanda.idZona%type,
    i_data IN TFGAdmin.demanda.data%type,
    i_quantitat IN TFGAdmin.demanda.quantitat%type,
    RSP OUT varchar
)
IS
    parametres varchar(500);
    registreDemandaAnual number(1);
BEGIN
    RSP:='OK';
    parametres:=i_idProducte||','||i_idZona||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS')||','||i_quantitat;
    --Insereix el registre de demanda
    INSERT INTO Demanda VALUES(i_idProducte,i_idZona,i_data,i_quantitat);
    
    /*demanda anual*/
    --Guarda el nombre de registres de demanda anual
    SELECT COUNT(*) INTO registreDemandaAnual
    FROM demandaAnual
    WHERE any_=EXTRACT(YEAR FROM i_data) AND idProducte=i_idProducte;
    --Si no hi ha registre
    IF registreDemandaAnual=0 THEN
        --Crea un registre amb un comptador a 1 als dies de demanda
        INSERT INTO demandaAnual VALUES(EXTRACT(YEAR FROM i_data),i_idProducte,i_quantitat,1);
    --Si hi ha registre
    ELSE
        --Suma la nova quantitat i puja una unitat el comptador de dies
        UPDATE demandaAnual SET quantitat=quantitat+i_quantitat,quantitatDies=quantitatDies+1
        WHERE any_=EXTRACT(YEAR FROM i_data) AND idProducte=i_idProducte;
    END IF;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que esborra un registre de demanda
Demana el producte, la zona i la data de la demanda (claus)
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE esborrarDemanda(
    i_idProducte IN TFGAdmin.demanda.idProducte%type,
    i_idZona IN TFGAdmin.demanda.idZona%type,
    i_data IN TFGAdmin.demanda.data%type,
    RSP OUT varchar
)
IS
    parametres varchar(500);
    quantitatAnterior TFGAdmin.demanda.quantitat%type;
BEGIN
    RSP:='OK';
    parametres:=i_idProducte||','||i_idZona||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS');
    --Guarda la quantitat anterior de demanda del registre a esborrar
    SELECT quantitat INTO quantitatAnterior
    FROM demanda
    WHERE idProducte=i_idProducte AND idZona=i_idZona AND data=i_data;
    --Esborra el registre de demanda
    DELETE demanda WHERE idProducte=i_idProducte AND idZona=i_idZona AND data=i_data;
    
    /*demanda anual*/
    --Treu la quantitat i resta una unitat al comptador de dies de demanda anual
    UPDATE demandaAnual SET quantitat=quantitat-quantitatAnterior,quantitatDies=quantitatDies-1
    WHERE any_=EXTRACT(YEAR FROM i_data) AND idProducte=i_idProducte;
    --Esborra els registres a 0
    DELETE demandaAnual WHERE quantitat=0;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que modifica un registre de demanda
Demana el producte, la zona, la data i la quantitat de la demanda
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE modificarDemanda(
    i_idProducte IN TFGAdmin.demanda.idProducte%type,
    i_idZona IN TFGAdmin.demanda.idZona%type,
    i_data IN TFGAdmin.demanda.data%type,
    i_quantitat IN TFGAdmin.demanda.quantitat%type,
    RSP OUT varchar
)
IS
    parametres varchar(500);
    quantitatAnterior TFGAdmin.demanda.quantitat%type;
BEGIN
    RSP:='OK';
    parametres:=i_idProducte||','||i_idZona||','||TO_CHAR(i_data,'DD-MM-YYYY HH24:MI:SS')||','||i_quantitat;
    --guarda la quantitat anterior del registre a modificar
    SELECT quantitat INTO quantitatAnterior
    FROM demanda
    WHERE idProducte=i_idProducte AND idZona=i_idZona AND data=i_data;
    --Modifica el registre de demanda
    UPDATE demanda SET quantitat=i_quantitat WHERE idProducte=i_idProducte AND idZona=i_idZona AND data=i_data;
    
    /*demanda anual*/
    --Actualitza el registre de demanda anual treient l'anterior quantitat i sumant la nova
    UPDATE demandaAnual SET quantitat=quantitat-quantitatAnterior+i_quantitat
    WHERE any_=EXTRACT(YEAR FROM i_data) AND idProducte=i_idProducte;
    --Esborra els registres a 0
    DELETE demandaAnual WHERE quantitat=0;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que crea un registre de netejament
Demana el grup de neteja, la lnia netejada, la data d'inici i la data de fi del netejament
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE crearNetejament(
    i_idNeteja TFGAdmin.netejament.idNeteja%type,
    i_idLinia TFGAdmin.netejament.idLinia%type,
    i_dataInici TFGAdmin.netejament.dataInici%type,
    i_dataFi TFGAdmin.netejament.dataFi%type,
    RSP OUT varchar
)
IS
    parametres varchar(500);
    existeixNetejaAnual number(1);
BEGIN
    RSP:='OK';
    parametres:=i_idNeteja||','||i_idLinia||','||TO_CHAR(i_dataInici,'DD-MM-YYYY HH24:MI:SS')||','||TO_CHAR(i_dataFi,'DD-MM-YYYY HH24:MI:SS');
    
    /*Neteja anual*/
    --Guarda els registres de neteja anual d'aquesta lnia
    SELECT COUNT(any_) INTO existeixNetejaAnual
    FROM netejaAnual
    WHERE any_=EXTRACT(YEAR FROM i_dataFi) AND idLinia=i_idLinia;
    --Si no hi ha registre
    IF existeixNetejaAnual=0 THEN
        --Insereix el registre amb les hores de neteja
        INSERT INTO netejaAnual VALUES(EXTRACT(YEAR FROM i_dataFi),i_idLinia,(i_dataFi-i_dataInici)*24);
    --Si hi ha registre
    ELSE
        --Suma les hores de neteja a la lnia
        UPDATE netejaAnual SET hores=hores+(i_dataFi-i_dataInici)*24 WHERE any_=EXTRACT(YEAR FROM i_dataFi) AND idLinia=i_idLinia;
    END IF;
    
    /*Netejament*/
    --Insereix el registre de netejament
    INSERT INTO netejament VALUES(i_idNeteja,i_idLinia,i_dataInici,i_dataFi);
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Procediment que modifica un registre de netejament
Demana el grup de neteja, la lnia netejada, la data d'inici i la data de fi del netejament
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE modificarNetejament(
    i_idNeteja TFGAdmin.netejament.idNeteja%type,
    i_idLinia TFGAdmin.netejament.idLinia%type,
    i_dataInici TFGAdmin.netejament.dataInici%type,
    i_dataFi TFGAdmin.netejament.dataFi%type,
    RSP OUT varchar
)
IS
    dataFiAnterior TFGAdmin.netejament.dataFi%type;
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_idNeteja||','||i_idLinia||','||TO_CHAR(i_dataInici,'DD-MM-YYYY HH24:MI:SS')||','||TO_CHAR(i_dataFi,'DD-MM-YYYY HH24:MI:SS');
    --Guarda l'anterior data de fi del registre a modificar
    SELECT dataFi INTO dataFiAnterior
    FROM netejament
    WHERE idNeteja=i_idNeteja AND idLinia=i_idLinia AND dataInici=i_dataInici;
    
    /*Neteja anual*/
    --Treu les hores anteriors i suma les noves hores (com que la data d'inici s part de la clau, noms poden modificar la data fi)
    UPDATE netejaAnual SET hores=hores-(dataFiAnterior-i_dataInici)*24+(i_dataFi-i_dataInici)*24 WHERE any_=EXTRACT(YEAR FROM i_dataFi) AND idLinia=i_idLinia;
    --Esborra els registres a 0
    DELETE netejaAnual WHERE hores=0;
    
    /*Netejament*/
    --Actualitza el registre de netejament
    UPDATE netejament SET dataFi=i_dataFi WHERE idNeteja=i_idNeteja AND idLinia=i_idLinia AND dataInici=i_dataInici;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;

/*
Esborra un registre de netejament
Demana el grup de neteja, la lnia netejada i la data d'inici (claus)
Retorna RSP variable d'errors
*/
CREATE OR REPLACE PROCEDURE esborrarNetejament(
    i_idNeteja TFGAdmin.netejament.idNeteja%type,
    i_idLinia TFGAdmin.netejament.idLinia%type,
    i_dataInici TFGAdmin.netejament.dataInici%type,
    RSP OUT varchar
)
IS
    dataFiAnterior TFGAdmin.netejament.dataFi%type;
    parametres varchar(500);
BEGIN
    RSP:='OK';
    parametres:=i_idNeteja||','||i_idLinia||','||TO_CHAR(i_dataInici,'DD-MM-YYYY HH24:MI:SS');
    --Guarda la data fi anterior al registre a esborrar
    SELECT dataFi INTO dataFiAnterior
    FROM netejament
    WHERE idNeteja=i_idNeteja AND idLinia=i_idLinia AND dataInici=i_dataInici;
    
    /*Neteja anual*/
    --Resta les hores anteriors
    UPDATE netejaAnual SET hores=hores-(dataFiAnterior-i_dataInici)*24 WHERE any_=EXTRACT(YEAR FROM dataFiAnterior) AND idLinia=i_idLinia;
    --Esborra els registres a 0
    DELETE netejaAnual WHERE hores=0;
    
    /*Netejament*/
    --Esborra el registre de netejament
    DELETE netejament WHERE idNeteja=i_idNeteja AND idLinia=i_idLinia AND dataInici=i_dataInici;
    
    crearLOG($$PLSQL_UNIT,parametres,RSP);
EXCEPTION
--Per qualsevol altra excepci
WHEN OTHERS THEN
     --Guarda el error a RSP
    RSP:='ERROR'|| ':' ||SQLERRM;
    --Crea el log amb el nom del procediment, parmetre i variable d'error
    crearLOG($$PLSQL_UNIT,parametres,RSP);
END;