CREATE TABLE TFGAdmin.envas(
    idEnvas varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    capacitat integer NOT NULL,
    CONSTRAINT envasPK PRIMARY KEY (idEnvas)
);

CREATE TABLE TFGAdmin.producte(
    idProducte varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    idEnvas varchar(50) NOT NULL,
    CONSTRAINT productePK PRIMARY KEY (idProducte),
    CONSTRAINT producteEnvasFK FOREIGN KEY (idEnvas) REFERENCES TFGAdmin.envas (idEnvas)
);

CREATE TABLE TFGAdmin.component(
    idComponent varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    CONSTRAINT componentPK PRIMARY KEY (idComponent)
);

CREATE TABLE TFGAdmin.zonaDistribucio(
    idZona varchar(15) NOT NULL,
    nomZona varchar(80) NOT NULL,
    descripcio varchar(200) NOT NULL,
    CONSTRAINT zonaDistribucioPK PRIMARY KEY (idZona)
);

CREATE TABLE TFGAdmin.sistemaEmbalatge(
    idSistema varchar(15) NOT NULL,
    idProducte varchar(200) NOT NULL,
    descripcio varchar(200) NOT NULL,
    quantitat integer NOT NULL,
    CONSTRAINT sistemaEmbalatgePK PRIMARY KEY (idSistema),
    CONSTRAINT sistemaEmbalatgeProducte FOREIGN KEY (idProducte) REFERENCES TFGAdmin.producte (idProducte)
);

CREATE TABLE TFGAdmin.tipusEmpresa(
    idTipus char(2) NOT NULL,
    descripcio varchar(200) NOT NULL,
    CONSTRAINT tipusEmpresaPK PRIMARY KEY (idTipus)
);

CREATE TABLE TFGAdmin.empresa(
    idEmpresa varchar(15) NOT NULL,
    CIF varchar(20) NOT NULL,
    nomComercial varchar(50) NOT NULL,
    nomFiscal varchar(50) NOT NULL,
    adreca varchar(100) NOT NULL,
    telefon varchar(15) NOT NULL,
    idTipus char(2) NOT NULL,
    idZona varchar(15) NULL,
    CONSTRAINT empresaPK PRIMARY KEY (idEmpresa),
    CONSTRAINT empresaTipusFK FOREIGN KEY (idTipus) REFERENCES TFGAdmin.tipusEmpresa (idTipus),
    CONSTRAINT empresaZonaFK FOREIGN KEY (idZona) REFERENCES TFGAdmin.zonaDistribucio (idZona),
    CONSTRAINT empresaCIFUnic UNIQUE (CIF)
);

CREATE TABLE TFGAdmin.empleat(
    idEmpleat varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    CONSTRAINT empleatPK PRIMARY KEY (idEmpleat)
);

CREATE TABLE TFGAdmin.fabrica(
    idFabrica varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    adreca varchar(100) NOT NULL,
    telefon varchar(15) NOT NULL,
    CONSTRAINT fabricaPK PRIMARY KEY (idFabrica)
);

CREATE TABLE TFGAdmin.liniaFabricacio(
    idLinia varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    llargada number(10,2) NOT NULL,
    idResponsable varchar(15) NOT NULL,
    idFabrica varchar(15) NOT NULL,
    CONSTRAINT liniaFabricacioPK PRIMARY KEY (idLinia),
    CONSTRAINT liniaFabricacioResponsableFK FOREIGN KEY (idResponsable) REFERENCES TFGAdmin.empleat (idEmpleat), 
    CONSTRAINT liniaFabricacioFabricaFK FOREIGN KEY (idFabrica) REFERENCES TFGAdmin.fabrica (idFabrica),
    CONSTRAINT llargadaPositiva CHECK (llargada>0)
);

CREATE TABLE TFGAdmin.neteja(
    idNeteja varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    CONSTRAINT netejaPK PRIMARY KEY (idNeteja)
);

CREATE TABLE TFGAdmin.torn(
    idTorn varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    idResponsable varchar(15) NOT NULL,
    horaInici varchar(8) NOT NULL,
    horaFi varchar(8) NOT NULL,
    capacitatPersones INTEGER NOT NULL,
    idLinia varchar(15) NOT NULL,
    CONSTRAINT tornPK PRIMARY KEY (idTorn),
    CONSTRAINT tornResponsableFK FOREIGN KEY (idResponsable) REFERENCES TFGAdmin.empleat (idEmpleat),
    CONSTRAINT tornLiniaFK FOREIGN KEY (idLinia) REFERENCES TFGAdmin.liniaFabricacio (idLinia)
);

CREATE TABLE transport(
    idTransport varchar(15) NOT NULL,
    tipus varchar(50) NOT NULL,
    descripcio varchar(200) NOT NULL,
    capacitat number(10,2) NOT NULL,
    disponibilitat varchar(50) NOT NULL,
    CONSTRAINT transportPK PRIMARY KEY (idTransport),
    CONSTRAINT transportCapacitatPositiva CHECK (capacitat>0)
);

CREATE TABLE magatzem(
    idMagatzem varchar(15) NOT NULL,
    descripcio varchar(200) NOT NULL,
    adreca varchar(100) NOT NULL,
    capacitat number(10,2) NOT NULL,
    CONSTRAINT magatzemPK PRIMARY KEY (idMagatzem),
    CONSTRAINT magatzemCapacitatPositiva CHECK (capacitat>0)
);

CREATE TABLE compost(
    idProducte varchar(15) NOT NULL,
    idComponent varchar(15) NOT NULL,
    quantitat integer NOT NULL,
    unitatMesura char(2) NOT NULL,
    CONSTRAINT compostPK PRIMARY KEY (idProducte,idComponent),
    CONSTRAINT compostProducteFK FOREIGN KEY (idProducte) REFERENCES TFGAdmin.producte (idProducte), 
    CONSTRAINT compostComponentFK FOREIGN KEY (idComponent) REFERENCES TFGAdmin.component (idComponent)
);

CREATE TABLE demanda(
    idProducte varchar(15) NOT NULL,
    idZona varchar(15) NOT NULL,
    data DATE NOT NULL,
    quantitat integer NOT NULL,
    CONSTRAINT demandaPK PRIMARY KEY (idProducte,idZona,data),
    CONSTRAINT demandaProducteFK FOREIGN KEY (idProducte) REFERENCES TFGAdmin.producte (idProducte), 
    CONSTRAINT demandaZonaFK FOREIGN KEY (idZona) REFERENCES TFGAdmin.zonaDistribucio (idZona),
    CONSTRAINT demandaQuantitatPositiva CHECK (quantitat>0)
); 

CREATE TABLE acces(
    idMagatzem varchar(15) NOT NULL,
    idEmpresa varchar(15) NOT NULL,
    horari varchar(50) NOT NULL,
    CONSTRAINT accesPK PRIMARY KEY (idMagatzem,idEmpresa),
    CONSTRAINT accesMagatzemFK FOREIGN KEY (idMagatzem) REFERENCES TFGAdmin.magatzem (idMagatzem), 
    CONSTRAINT accesEmpresaFK FOREIGN KEY (idEmpresa) REFERENCES TFGAdmin.empresa (idEmpresa)
); 

CREATE TABLE produccio(
    idProducte varchar(15) NOT NULL,
    idLinia varchar(15) NOT NULL,
    dataInici DATE NOT NULL,
    dataFi DATE NOT NULL,
    quantitatFabricada integer,
    CONSTRAINT produccioPK PRIMARY KEY (idProducte,idLinia,dataInici),
    CONSTRAINT produccioProducteFK FOREIGN KEY (idProducte) REFERENCES TFGAdmin.producte (idProducte),
    CONSTRAINT produccioLiniaFK FOREIGN KEY (idLinia) REFERENCES TFGAdmin.liniaFabricacio (idLinia)
); 

CREATE TABLE netejament(
    idNeteja varchar(15) NOT NULL,
    idLinia varchar(15) NOT NULL,
    dataInici DATE NOT NULL,
    dataFi DATE NOT NULL,
    CONSTRAINT netejamentPK PRIMARY KEY (idNeteja,idLinia,dataInici),
    CONSTRAINT netejamentLiniaFK FOREIGN KEY (idLinia) REFERENCES TFGAdmin.liniaFabricacio (idLinia),
    CONSTRAINT netejamentNetejaFK FOREIGN KEY (idNeteja) REFERENCES TFGAdmin.neteja (idNeteja)
); 

CREATE TABLE assignacio(
    idEmpleat varchar(15) NOT NULL,
    idTorn varchar(15) NOT NULL,
    data DATE NOT NULL,
    CONSTRAINT assignacioPK PRIMARY KEY (idEmpleat,idTorn,data),
    CONSTRAINT assignacioEmpleatFK FOREIGN KEY (idEmpleat) REFERENCES TFGAdmin.empleat (idEmpleat),
    CONSTRAINT assignacioTornFK FOREIGN KEY (idTorn) REFERENCES TFGAdmin.torn (idTorn)
); 

CREATE TABLE stock(
    idSistema varchar(15) NOT NULL,
    idMagatzem varchar(15) NOT NULL,
    data DATE NOT NULL,
    volum decimal(10,2) NOT NULL,
    quantitat integer NOT NULL,
    idTransport varchar(15) NOT NULL,
    idEmpresa varchar(15) NULL,
    CONSTRAINT stockPK PRIMARY KEY (idSistema,idMagatzem,data),
    CONSTRAINT stockSistemaFK FOREIGN KEY (idSistema) REFERENCES TFGAdmin.sistemaEmbalatge (idSistema),
    CONSTRAINT stockMagatzemFK FOREIGN KEY (idMagatzem) REFERENCES TFGAdmin.magatzem (idMagatzem),
    CONSTRAINT stockTransportFK FOREIGN KEY (idTransport) REFERENCES TFGAdmin.transport (idTransport),
    CONSTRAINT stockEmpresaFK FOREIGN KEY (idEmpresa) REFERENCES TFGAdmin.empresa (idEmpresa)
); 